/*
 * phc -- the open source PHP compiler
 * See license/README.license for licensing information
 *
 * Error handling 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <phc/lib/error.h>

// printf format statements are supported

static char* error_msg[] = {
	NULL, /* start at error code 1 */
	/* ERR_BISON */ "%s",
	/* ERR_EXPECT_VAR */ "Variable name expected",
	/* ERR_FOREACH_KEY_REF */ "Key element cannot be a reference",
	/* ERR_ABSTRACT_VARS */ "Cannot declare variables to be abstract",
	/* ERR_FINAL_VARS */ "The final modifier is only allowed for methods",
	/* ERR_XML_PARSE */ "Could not parse the XML (%s)",
	/* ERR_NO_XML_SUPPORT */ "XML support not built-in",
	/* ERR_PLUGIN_NOT_FOUND */ "Could not find the plugin (tried \"%s\", \"%s\" and \"%s\")",
	/* ERR_PLUGIN_INVALID */ "Invalid plugin (%s)",
	/* ERR_FILE_NOT_FOUND */ "File not found",
};

static char* warning_msg[] = {
	NULL, /* start at error code 1 */
	/* WARNING_INCLUDE_FAILED */ "File %s could not be included",
	/* WARNING_INCLUDE_RETURN */ "Returning values from included scripts is not supported in file %s", 
	/* WARNING_XML */ "XML error: %s",
	/* WARNING_UNKNOWN_ATTRIBUTE */ "Unknown attribute '%s'",
};

// Print the error message and quit
void error(int code, String* filename, int line, ...)
{
	va_list argp;
	
	if(filename)
		fprintf(stderr, "%s:", filename->c_str());
	
	if(line > 0)
		fprintf(stderr, "%d:", line);

	fprintf(stderr, " error: ");

	va_start(argp, line);
	vfprintf(stderr, error_msg[code], argp);
	va_end(argp);
	
	fprintf(stderr, "\n");
	exit(code);
}

// Print the warning and continue 
void warning(int code, String* filename, int line, ...)
{
	va_list argp;
	
	if(filename)
		fprintf(stderr, "%s:", filename->c_str());
	
	if(line > 0)
		fprintf(stderr, "%d:", line);

	fprintf(stderr, " warning: ");

	va_start(argp, line);
	vfprintf(stderr, warning_msg[code], argp);
	va_end(argp);
	
	fprintf(stderr, "\n");
}
